@@ -22,12 +22,15 @@
  *
  */
 
+#include <config.h>
+
 #include <math.h>
 #include "cc-ua-panel.h"
 
 #include <gconf/gconf-client.h>
 
-#include "gconf-property-editor.h"
+#include "eggaccelerators.h"
+#include "libgnome-control-center/gconf-property-editor.h"
 
 #define WID(b, w) (GtkWidget *) gtk_builder_get_object (b, w)
 
@@ -44,7 +47,8 @@ struct _CcUaPanelPrivate
   GSettings *interface_settings;
   GSettings *kb_settings;
   GSettings *mouse_settings;
-  GSettings *font_settings;
+  GSettings *application_settings;
+  GSettings *mediakeys_settings;
 
   GSList *notify_list;
 };
@@ -126,10 +130,16 @@ cc_ua_panel_dispose (GObject *object)
       priv->mouse_settings = NULL;
     }
 
-  if (priv->font_settings)
+  if (priv->application_settings)
     {
-      g_object_unref (priv->font_settings);
-      priv->font_settings = NULL;
+      g_object_unref (priv->application_settings);
+      priv->application_settings = NULL;
+    }
+
+  if (priv->mediakeys_settings)
+    {
+      g_object_unref (priv->mediakeys_settings);
+      priv->mediakeys_settings = NULL;
     }
 
   G_OBJECT_CLASS (cc_ua_panel_parent_class)->dispose (object);
@@ -159,6 +169,11 @@ cc_ua_panel_class_finalize (CcUaPanelClass *klass)
 {
 }
 
+static gchar *typing_assistant_section[] = {
+    "typing_assistant_preferences_button",
+    NULL
+};
+
 static gchar *sticky_keys_section[] = {
     "typing_sticky_keys_disable_two_keys_checkbutton",
     "typing_sticky_keys_beep_modifier_checkbutton",
@@ -195,17 +210,25 @@ static gchar *visual_alerts_section[] = {
     NULL
 };
 
+static gchar *shortcuts_section[] = {
+    "stickykeys_frame",
+    "slowkeys_frame",
+    "bouncekeys_frame",
+    NULL
+};
+
 static void
-cc_ua_panel_section_toggled (GtkToggleButton *button,
-                             GtkBuilder      *builder)
+cc_ua_panel_section_switched (GObject    *object,
+                              GParamSpec *pspec,
+                              GtkBuilder *builder)
 {
   GtkWidget *w;
   gboolean enabled;
   gchar **widgets, **s;
 
-  widgets = g_object_get_data (G_OBJECT (button), "section-widgets");
+  widgets = g_object_get_data (object, "section-widgets");
 
-  enabled = gtk_toggle_button_get_active (button);
+  g_object_get (object, "active", &enabled, NULL);
 
   for (s = widgets; *s; s++)
     {
@@ -225,8 +248,8 @@ settings_on_off_editor_new (CcUaPanelPrivate  *priv,
   if (section)
     {
       g_object_set_data (G_OBJECT (widget), "section-widgets", section);
-      g_signal_connect (widget, "toggled",
-                        G_CALLBACK (cc_ua_panel_section_toggled),
+      g_signal_connect (widget, "notify::active",
+                        G_CALLBACK (cc_ua_panel_section_switched),
                         priv->builder);
     }
 
@@ -234,35 +257,17 @@ settings_on_off_editor_new (CcUaPanelPrivate  *priv,
   g_settings_bind (settings, key, widget, "active", G_SETTINGS_BIND_DEFAULT);
 }
 
-static GConfValue*
-cc_ua_panel_toggle_radios (GConfPropertyEditor *peditor,
+static GConfValue *
+cc_ua_panel_toggle_switch (GConfPropertyEditor *peditor,
                            const GConfValue    *value)
 {
-  GtkWidget *radio;
+  GtkWidget *sw;
   gboolean enabled;
 
   enabled = gconf_value_get_bool (value);
-  radio = (GtkWidget*) gconf_property_editor_get_ui_control (peditor);
+  sw = (GtkWidget*) gconf_property_editor_get_ui_control (peditor);
 
-  if (!enabled)
-    {
-      GSList *list, *l;
-
-      list = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
-
-      if (list)
-        {
-          /* activate the "off" button */
-          for (l = list; l; l = l->next)
-            {
-              if (l->data == radio)
-                continue;
-
-              gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (l->data),
-                                            TRUE);
-            }
-        }
-    }
+  gtk_switch_set_active (GTK_SWITCH (sw), enabled);
 
   return gconf_value_copy (value);
 }
@@ -279,14 +284,14 @@ gconf_on_off_peditor_new (CcUaPanelPrivate  *priv,
   if (section)
     {
       g_object_set_data (G_OBJECT (widget), "section-widgets", section);
-      g_signal_connect (widget, "toggled",
-                        G_CALLBACK (cc_ua_panel_section_toggled),
+      g_signal_connect (widget, "notify::active",
+                        G_CALLBACK (cc_ua_panel_section_switched),
                         priv->builder);
     }
 
   /* set up the boolean editor */
-  peditor = gconf_peditor_new_boolean (NULL, key, widget, NULL);
-  g_object_set (peditor, "conv-to-widget-cb", cc_ua_panel_toggle_radios, NULL);
+  peditor = gconf_peditor_new_switch (NULL, key, widget, NULL);
+  g_object_set (peditor, "conv-to-widget-cb", cc_ua_panel_toggle_switch, NULL);
 
   /* emit the notify on the key, so that the conv-to-widget-cb callback is run */
   gconf_client_notify (priv->client, key);
@@ -298,79 +303,12 @@ gconf_on_off_peditor_new (CcUaPanelPrivate  *priv,
 #define CONTRAST_MODEL_THEME_COLUMN 3
 #define DPI_MODEL_FACTOR_COLUMN 2
 
-/* The following two functions taken from gsd-a11y-preferences-dialog.c
- *
- * Copyright (C)  2008 William Jon McCann <jmccann@redhat.com>
- *
- * Licensed under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- */
-/* X servers sometimes lie about the screen's physical dimensions, so we cannot
- * compute an accurate DPI value.  When this happens, the user gets fonts that
- * are too huge or too tiny.  So, we see what the server returns:  if it reports
- * something outside of the range [DPI_LOW_REASONABLE_VALUE,
- * DPI_HIGH_REASONABLE_VALUE], then we assume that it is lying and we use
- * DPI_FALLBACK instead.
- *
- * See get_dpi_from_x_server() below, and also
- * https://bugzilla.novell.com/show_bug.cgi?id=217790
- */
-#define DPI_LOW_REASONABLE_VALUE 50
-#define DPI_HIGH_REASONABLE_VALUE 500
-#define DPI_DEFAULT        96
-
-static gdouble
-dpi_from_pixels_and_mm (gint pixels,
-                        gint mm)
-{
-  if (mm >= 1)
-    return pixels / (mm / 25.4);
-  else
-    return 0;
-}
-
-static gdouble
-get_dpi_from_x_server ()
-{
-  GdkScreen *screen;
-  gdouble dpi;
-
-  screen = gdk_screen_get_default ();
-
-  if (screen)
-    {
-      gdouble width_dpi, height_dpi;
-
-      width_dpi = dpi_from_pixels_and_mm (gdk_screen_get_width (screen),
-                                          gdk_screen_get_width_mm (screen));
-      height_dpi = dpi_from_pixels_and_mm (gdk_screen_get_height (screen),
-                                           gdk_screen_get_height_mm (screen));
-
-      if (width_dpi < DPI_LOW_REASONABLE_VALUE
-          || width_dpi > DPI_HIGH_REASONABLE_VALUE
-          || height_dpi < DPI_LOW_REASONABLE_VALUE
-          || height_dpi > DPI_HIGH_REASONABLE_VALUE)
-        {
-          dpi = DPI_DEFAULT;
-        }
-      else
-        {
-          dpi = (width_dpi + height_dpi) / 2.0;
-        }
-    }
-  else
-    dpi = DPI_DEFAULT;
-
-  return dpi;
-}
-
-static void dpi_combo_box_changed (GtkComboBox *box, CcUaPanel *panel);
+static void text_scaling_factor_combo_box_changed (GtkComboBox *box, CcUaPanel *panel);
 
 static void
-dpi_notify_cb (GSettings   *settings,
-               const gchar *key,
-               CcUaPanel   *panel)
+text_scaling_factor_notify_cb (GSettings   *settings,
+			       const gchar *key,
+			       CcUaPanel   *panel)
 {
   CcUaPanelPrivate *priv = panel->priv;
   GtkTreeIter iter;
@@ -378,11 +316,10 @@ dpi_notify_cb (GSettings   *settings,
   GtkWidget *combo;
   gboolean valid;
   gdouble conf_value;
-  gdouble x_dpi;
   GtkTreeIter best;
   gdouble distance;
 
-  if (!g_str_equal (key, "dpi"))
+  if (!g_str_equal (key, "text-scaling-factor"))
     return;
 
   conf_value = g_settings_get_double (settings, key);
@@ -390,9 +327,6 @@ dpi_notify_cb (GSettings   *settings,
   combo = WID (priv->builder, "seeing_text_size_combobox");
   model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
 
-  /* get current value from screen */
-  x_dpi = get_dpi_from_x_server ();
-
   /* find the closest match in the combobox model */
   distance = 1e6;
   valid = gtk_tree_model_get_iter_first (model, &iter);
@@ -405,7 +339,7 @@ dpi_notify_cb (GSettings   *settings,
                           DPI_MODEL_FACTOR_COLUMN, &factor,
                           -1);
 
-      d = fabs (conf_value - factor * x_dpi);
+      d = fabs (conf_value - factor);
       if (d < distance)
         {
           best = iter;
@@ -415,14 +349,14 @@ dpi_notify_cb (GSettings   *settings,
       valid = gtk_tree_model_iter_next (model, &iter);
     }
 
-  g_signal_handlers_block_by_func (combo, dpi_combo_box_changed, panel);
+  g_signal_handlers_block_by_func (combo, text_scaling_factor_combo_box_changed, panel);
   gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &best);
-  g_signal_handlers_unblock_by_func (combo, dpi_combo_box_changed, panel);
+  g_signal_handlers_unblock_by_func (combo, text_scaling_factor_combo_box_changed, panel);
 }
 
 static void
-dpi_combo_box_changed (GtkComboBox *box,
-                       CcUaPanel *panel)
+text_scaling_factor_combo_box_changed (GtkComboBox *box,
+				       CcUaPanel *panel)
 {
   CcUaPanelPrivate *priv = panel->priv;
   GtkTreeIter iter;
@@ -435,16 +369,9 @@ dpi_combo_box_changed (GtkComboBox *box,
                       -1);
 
   if (factor == 1.0)
-    g_settings_reset (priv->font_settings, "dpi");
+    g_settings_reset (priv->interface_settings, "text-scaling-factor");
   else
-    {
-      gdouble x_dpi, u_dpi;
-
-      x_dpi = get_dpi_from_x_server ();
-      u_dpi = (gdouble) factor * x_dpi;
-
-      g_settings_set_double (priv->font_settings, "dpi", u_dpi);
-    }
+    g_settings_set_double (priv->interface_settings, "text-scaling-factor", factor);
 }
 
 
@@ -525,22 +452,73 @@ contrast_combobox_changed_cb (GtkComboBox *box,
   g_free (theme_name);
 }
 
+static void
+cc_ua_panel_set_shortcut_label (CcUaPanel  *self,
+				const char *label,
+				const char *key)
+{
+	GtkWidget *widget;
+	char *value;
+	char *text;
+	guint accel_key, keycode;
+	EggVirtualModifierType mods;
+
+	widget = WID (self->priv->builder, label);
+	value = g_settings_get_string (self->priv->mediakeys_settings, key);
+
+	if (value == NULL || *value == '\0') {
+		gtk_label_set_text (GTK_LABEL (widget), _("No shortcut set"));
+		g_free (value);
+		return;
+	}
+	if (egg_accelerator_parse_virtual (value, &accel_key, &keycode, &mods) == FALSE) {
+		gtk_label_set_text (GTK_LABEL (widget), _("No shortcut set"));
+		g_free (value);
+		g_warning ("Failed to parse keyboard shortcut: '%s'", value);
+		return;
+	}
+	g_free (value);
+
+	text = egg_virtual_accelerator_label (accel_key, keycode, mods);
+	gtk_label_set_text (GTK_LABEL (widget), text);
+	g_free (text);
+}
+
 static void
 cc_ua_panel_init_seeing (CcUaPanel *self)
 {
   CcUaPanelPrivate *priv = self->priv;
 
-  g_signal_connect (priv->font_settings, "changed", G_CALLBACK (dpi_notify_cb), self);
+  text_scaling_factor_notify_cb (priv->interface_settings, "text-scaling-factor", self);
+  g_signal_connect (priv->interface_settings, "changed::text-scaling-factor", G_CALLBACK (text_scaling_factor_notify_cb), self);
 
   g_signal_connect (WID (priv->builder, "seeing_contrast_combobox"), "changed",
                     G_CALLBACK (contrast_combobox_changed_cb), self);
 
   g_signal_connect (WID (priv->builder, "seeing_text_size_combobox"), "changed",
-                    G_CALLBACK (dpi_combo_box_changed), self);
+                    G_CALLBACK (text_scaling_factor_combo_box_changed), self);
 
   g_settings_bind (priv->kb_settings, "togglekeys-enable",
                    WID (priv->builder, "seeing_enable_toggle_keys_checkbutton"), "active",
                    G_SETTINGS_BIND_DEFAULT);
+
+  settings_on_off_editor_new (priv, priv->application_settings,
+                              "screen-magnifier-enabled",
+                              WID (priv->builder, "seeing_zoom_switch"),
+                              NULL);
+
+  settings_on_off_editor_new (priv, priv->application_settings,
+                              "screen-reader-enabled",
+                              WID (priv->builder, "seeing_reader_switch"),
+                              NULL);
+
+  cc_ua_panel_set_shortcut_label (self, "seeing_contrast_toggle_keybinding_label", "toggle-contrast");
+  cc_ua_panel_set_shortcut_label (self, "seeing_increase_size_keybinding_label", "increase-text-size");
+  cc_ua_panel_set_shortcut_label (self, "seeing_decrease_size_keybinding_label", "decrease-text-size");
+  cc_ua_panel_set_shortcut_label (self, "seeing_zoom_enable_keybinding_label", "magnifier");
+  cc_ua_panel_set_shortcut_label (self, "seeing_zoom_in_keybinding_label", "magnifier-zoom-in");
+  cc_ua_panel_set_shortcut_label (self, "seeing_zoom_out_keybinding_label", "magnifier-zoom-out");
+  cc_ua_panel_set_shortcut_label (self, "seeing_reader_enable_keybinding_label", "screenreader");
 }
 
 
@@ -598,7 +576,7 @@ cc_ua_panel_init_hearing (CcUaPanel *self)
   gconf_client_add_dir (priv->client, "/apps/metacity/general",
                         GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
-  w = WID (priv->builder, "hearing_visual_alerts_on_radiobutton");
+  w = WID (priv->builder, "hearing_visual_alerts_switch");
   gconf_on_off_peditor_new (priv, "/apps/metacity/general/visual_bell",
                             w, visual_alerts_section);
 
@@ -645,22 +623,26 @@ cc_ua_panel_init_keyboard (CcUaPanel *self)
   CcUaPanelPrivate *priv = self->priv;
   GtkWidget *w;
 
+  /* Typing assistant (on-screen keyboard) */
+  w = WID (priv->builder, "typing_assistant_switch");
+  settings_on_off_editor_new (priv, priv->application_settings, "screen-keyboard-enabled", w, typing_assistant_section);
+
   /* enable shortcuts */
   w = WID (priv->builder, "typing_keyboard_toggle_checkbox");
-  g_settings_bind (priv->kb_settings, "enable", w, "active", G_SETTINGS_BIND_DEFAULT);
+  settings_on_off_editor_new (priv, priv->kb_settings, "enable", w, shortcuts_section);
 
   /* sticky keys */
-  w = WID (priv->builder, "typing_sticky_keys_on_radiobutton");
+  w = WID (priv->builder, "typing_sticky_keys_switch");
   settings_on_off_editor_new (priv, priv->kb_settings, "stickykeys-enable", w, sticky_keys_section);
 
   w = WID (priv->builder, "typing_sticky_keys_disable_two_keys_checkbutton");
-  g_settings_bind (priv->kb_settings, "stickykeys-two-key-off", w, "active", G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (priv->kb_settings, "stickykeys-two-key-off", w, "active", G_SETTINGS_BIND_NO_SENSITIVITY);
 
   w = WID (priv->builder, "typing_sticky_keys_beep_modifier_checkbutton");
-  g_settings_bind (priv->kb_settings, "stickykeys-modifier-beep", w, "active", G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (priv->kb_settings, "stickykeys-modifier-beep", w, "active", G_SETTINGS_BIND_NO_SENSITIVITY);
 
   /* slow keys */
-  w = WID (priv->builder, "typing_slow_keys_on_radiobutton");
+  w = WID (priv->builder, "typing_slow_keys_switch");
   settings_on_off_editor_new (priv, priv->kb_settings, "slowkeys-enable", w, slow_keys_section);
 
   w = WID (priv->builder, "typing_slowkeys_delay_scale");
@@ -678,7 +660,7 @@ cc_ua_panel_init_keyboard (CcUaPanel *self)
   g_settings_bind (priv->kb_settings, "slowkeys-beep-reject", w, "active", G_SETTINGS_BIND_DEFAULT);
 
   /* bounce keys */
-  w = WID (priv->builder, "typing_bounce_keys_on_radiobutton");
+  w = WID (priv->builder, "typing_bounce_keys_switch");
   settings_on_off_editor_new (priv, priv->kb_settings, "bouncekeys-enable", w, bounce_keys_section);
 
   w = WID (priv->builder, "typing_bouncekeys_delay_scale");
@@ -687,7 +669,7 @@ cc_ua_panel_init_keyboard (CcUaPanel *self)
                    G_SETTINGS_BIND_DEFAULT);
 
   w = WID (priv->builder, "typing_bounce_keys_beep_rejected_checkbutton");
-  g_settings_bind (priv->kb_settings, "bouncekeys-beep-reject", w, "active", G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (priv->kb_settings, "bouncekeys-beep-reject", w, "active", G_SETTINGS_BIND_NO_SENSITIVITY);
 
   g_signal_connect (WID (priv->builder, "typing_keyboard_preferences_button"),
                     "clicked",
@@ -712,11 +694,11 @@ cc_ua_panel_init_mouse (CcUaPanel *self)
   GtkWidget *w;
 
   /* mouse keys */
-  w = WID (priv->builder, "pointing_mouse_keys_on_radiobutton");
+  w = WID (priv->builder, "pointing_mouse_keys_switch");
   settings_on_off_editor_new (priv, priv->kb_settings, "mousekeys-enable", w, NULL);
 
   /* simulated secondary click */
-  w = WID (priv->builder, "pointing_second_click_on_radiobutton");
+  w = WID (priv->builder, "pointing_second_click_switch");
   settings_on_off_editor_new (priv, priv->mouse_settings, "secondary-click-enabled", w, secondary_click_section);
 
   w = WID (priv->builder, "pointing_secondary_click_delay_scale");
@@ -725,7 +707,7 @@ cc_ua_panel_init_mouse (CcUaPanel *self)
                    G_SETTINGS_BIND_DEFAULT);
 
   /* dwell click */
-  w = WID (priv->builder, "pointing_hover_click_on_radiobutton");
+  w = WID (priv->builder, "pointing_hover_click_switch");
   settings_on_off_editor_new (priv, priv->mouse_settings, "dwell-click-enabled", w, dwell_click_section);
 
   w = WID (priv->builder, "pointing_dwell_delay_scale");
@@ -788,7 +770,8 @@ cc_ua_panel_init (CcUaPanel *self)
 
   priv->kb_settings = g_settings_new ("org.gnome.desktop.a11y.keyboard");
   priv->mouse_settings = g_settings_new ("org.gnome.desktop.a11y.mouse");
-  priv->font_settings = g_settings_new ("org.gnome.settings-daemon.plugins.xsettings");
+  priv->application_settings = g_settings_new ("org.gnome.desktop.a11y.applications");
+  priv->mediakeys_settings = g_settings_new ("org.gnome.settings-daemon.plugins.media-keys");
 
   cc_ua_panel_init_keyboard (self);
   cc_ua_panel_init_mouse (self);
