@@ -117,6 +117,14 @@ get_selected_user (UmUserPanelPrivate *d)
         return NULL;
 }
 
+static char *
+get_name_col_str (UmUser *user)
+{
+        return g_markup_printf_escaped ("<b>%s</b>\n<i>%s</i>",
+                                        um_user_get_display_name (user),
+                                        um_account_type_get_name (um_user_get_account_type (user)));
+}
+
 static void
 user_added (UmUserManager *um, UmUser *user, UmUserPanelPrivate *d)
 {
@@ -129,7 +137,6 @@ user_added (UmUserManager *um, UmUser *user, UmUserPanelPrivate *d)
         gchar *text;
         GtkTreeSelection *selection;
         gint sort_key;
-        gchar *escaped_name;
         gboolean is_autologin;
 
         g_debug ("user added: %d %s\n", um_user_get_uid (user), um_user_get_real_name (user));
@@ -139,11 +146,7 @@ user_added (UmUserManager *um, UmUser *user, UmUserPanelPrivate *d)
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
 
         pixbuf = um_user_render_icon (user, TRUE, 48);
-        escaped_name = g_markup_escape_text (um_user_get_display_name (user), -1);
-        text = g_strdup_printf ("<b>%s</b>\n<i>%s</i>",
-                                escaped_name,
-                                um_account_type_get_name (um_user_get_account_type (user)));
-        g_free (escaped_name);
+        text = get_name_col_str (user);
 
         is_autologin = um_user_get_automatic_login (user);
 
@@ -270,9 +273,7 @@ user_changed (UmUserManager *um, UmUser *user, UmUserPanelPrivate *d)
                 gtk_tree_model_get (model, &iter, USER_COL, &current, -1);
                 if (current == user) {
                         pixbuf = um_user_render_icon (user, TRUE, 48);
-                        text = g_strdup_printf ("<b>%s</b>\n<i>%s</i>",
-                                                um_user_get_display_name (user),
-                                                um_account_type_get_name (um_user_get_account_type (user)));
+                        text = get_name_col_str (user);
                         is_autologin = um_user_get_automatic_login (user);
 
                         gtk_list_store_set (GTK_LIST_STORE (model), &iter,
@@ -831,9 +832,12 @@ users_loaded (UmUserManager     *manager,
                                                  _("Failed to contact the accounts service"));
                 gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                                           _("Please make sure that the AccountService is installed and enabled."));
-                g_signal_connect (dialog, "response",
-                                  G_CALLBACK (gtk_main_quit), NULL);
+                g_signal_connect_swapped (dialog, "response",
+                                          G_CALLBACK (gtk_widget_destroy),
+                                          dialog);
                 gtk_widget_show (dialog);
+
+                gtk_widget_set_sensitive (d->main_box, FALSE);
         }
 
         list = um_user_manager_list_users (d->um);
